@@ -4,6 +4,7 @@ Constants used by osxphotos
 
 import os.path
 from datetime import datetime
+from enum import Enum
 
 OSXPHOTOS_URL = "https://github.com/RhetTbull/osxphotos"
 
@@ -34,11 +35,12 @@ _PHOTOS_3_VERSION = "3301"
 
 # versions 5.0 and later have a different database structure
 _PHOTOS_4_VERSION = "4025"  # latest Mojove version on 10.14.6
-_PHOTOS_5_VERSION = "6000"  # seems to be current on 10.15.1 through 10.15.6
+_PHOTOS_5_VERSION = "6000"  # seems to be current on 10.15.1 through 10.15.7 (also Big Sur and Monterey which switch to model version)
 
 # Ranges for model version by Photos version
 _PHOTOS_5_MODEL_VERSION = [13000, 13999]
 _PHOTOS_6_MODEL_VERSION = [14000, 14999]
+_PHOTOS_7_MODEL_VERSION = [15000, 15999]  # Monterey developer preview is 15134
 
 # some table names differ between Photos 5 and Photos 6
 _DB_TABLE_NAMES = {
@@ -49,6 +51,10 @@ _DB_TABLE_NAMES = {
         "ALBUM_SORT_ORDER": "Z_26ASSETS.Z_FOK_34ASSETS",
         "IMPORT_FOK": "ZGENERICASSET.Z_FOK_IMPORTSESSION",
         "DEPTH_STATE": "ZGENERICASSET.ZDEPTHSTATES",
+        "UTI_ORIGINAL": "ZINTERNALRESOURCE.ZUNIFORMTYPEIDENTIFIER",
+        "ASSET_ALBUM_JOIN": "Z_26ASSETS.Z_26ALBUMS",
+        "ASSET_ALBUM_TABLE": "Z_26ASSETS",
+        "HDR_TYPE": "ZCUSTOMRENDEREDVALUE",
     },
     6: {
         "ASSET": "ZASSET",
@@ -57,6 +63,22 @@ _DB_TABLE_NAMES = {
         "ALBUM_SORT_ORDER": "Z_26ASSETS.Z_FOK_3ASSETS",
         "IMPORT_FOK": "null",
         "DEPTH_STATE": "ZASSET.ZDEPTHTYPE",
+        "UTI_ORIGINAL": "ZINTERNALRESOURCE.ZUNIFORMTYPEIDENTIFIER",
+        "ASSET_ALBUM_JOIN": "Z_26ASSETS.Z_26ALBUMS",
+        "ASSET_ALBUM_TABLE": "Z_26ASSETS",
+        "HDR_TYPE": "ZCUSTOMRENDEREDVALUE",
+    },
+    7: {
+        "ASSET": "ZASSET",
+        "KEYWORD_JOIN": "Z_1KEYWORDS.Z_38KEYWORDS",
+        "ALBUM_JOIN": "Z_27ASSETS.Z_3ASSETS",
+        "ALBUM_SORT_ORDER": "Z_27ASSETS.Z_FOK_3ASSETS",
+        "IMPORT_FOK": "null",
+        "DEPTH_STATE": "ZASSET.ZDEPTHTYPE",
+        "UTI_ORIGINAL": "ZINTERNALRESOURCE.ZCOMPACTUTI",
+        "ASSET_ALBUM_JOIN": "Z_27ASSETS.Z_27ALBUMS",
+        "ASSET_ALBUM_TABLE": "Z_27ASSETS",
+        "HDR_TYPE": "ZHDRTYPE",
     },
 }
 
@@ -70,6 +92,8 @@ _TESTED_OS_VERSIONS = [
     ("11", "0"),
     ("11", "1"),
     ("11", "2"),
+    ("11", "3"),
+    ("11", "4"),
 ]
 
 # Photos 5 has persons who are empty string if unidentified face
@@ -187,6 +211,9 @@ DEFAULT_EDITED_SUFFIX = "_edited"
 # Default suffix to add to original images
 DEFAULT_ORIGINAL_SUFFIX = ""
 
+# Default suffix to add to preview images
+DEFAULT_PREVIEW_SUFFIX = "_preview"
+
 # Colors for print CLI messages
 CLI_COLOR_ERROR = "red"
 CLI_COLOR_WARNING = "yellow"
@@ -201,12 +228,59 @@ EXTENDED_ATTRIBUTE_NAMES = [
     "authors",
     "comment",
     "copyright",
+    "creator",
     "description",
     "findercomment",
     "headline",
     "keywords",
+    "participants",
+    "projects",
+    "rating",
+    "subject",
+    "title",
+    "version",
 ]
 EXTENDED_ATTRIBUTE_NAMES_QUOTED = [f"'{x}'" for x in EXTENDED_ATTRIBUTE_NAMES]
 
 # name of export DB
-OSXPHOTOS_EXPORT_DB = ".osxphotos_export.db"
\ No newline at end of file
+OSXPHOTOS_EXPORT_DB = ".osxphotos_export.db"
+
+# bit flags for burst images ("burstPickType")
+BURST_NOT_SELECTED = 0b10  # 2: burst image is not selected
+BURST_DEFAULT_PICK = 0b100  # 4: burst image is the one Photos picked to be key image before any selections made
+BURST_SELECTED = 0b1000  # 8: burst image is selected
+BURST_KEY = 0b10000  # 16: burst image is the key photo (top of burst stack)
+BURST_UNKNOWN = 0b100000  # 32: this is almost always set with BURST_DEFAULT_PICK and never if BURST_DEFAULT_PICK is not set.  I think this has something to do with what algorithm Photos used to pick the default image
+
+LIVE_VIDEO_EXTENSIONS = [".mov"]
+
+# categories that --post-command can be used with; these map to ExportResults fields
+POST_COMMAND_CATEGORIES = {
+    "exported": "All exported files",
+    "new": "When used with '--update', all newly exported files",
+    "updated": "When used with '--update', all files which were previously exported but updated this time",
+    "skipped": "When used with '--update', all files which were skipped (because they were previously exported and didn't change)",
+    "missing": "All files which were not exported because they were missing from the Photos library",
+    "exif_updated": "When used with '--exiftool', all files on which exiftool updated the metadata",
+    "touched": "When used with '--touch-file', all files where the date was touched",
+    "converted_to_jpeg": "When used with '--convert-to-jpeg', all files which were converted to jpeg",
+    "sidecar_json_written": "When used with '--sidecar json', all JSON sidecar files which were written",
+    "sidecar_json_skipped": "When used with '--sidecar json' and '--update', all JSON sidecar files which were skipped",
+    "sidecar_exiftool_written": "When used with '--sidecar exiftool', all exiftool sidecar files which were written",
+    "sidecar_exiftool_skipped": "When used with '--sidecar exiftool' and '--update, all exiftool sidecar files which were skipped",
+    "sidecar_xmp_written": "When used with '--sidecar xmp', all XMP sidecar files which were written",
+    "sidecar_xmp_skipped": "When used with '--sidecar xmp' and '--update', all XMP sidecar files which were skipped",
+    "error": "All files which produced an error during export",
+    # "deleted_files": "When used with '--cleanup', all files deleted during the export",
+    # "deleted_directories": "When used with '--cleanup', all directories deleted during the export",
+}
+
+class AlbumSortOrder(Enum):
+    """Album Sort Order"""
+    UNKNOWN = 0
+    MANUAL = 1
+    NEWEST_FIRST = 2
+    OLDEST_FIRST = 3
+    TITLE = 5
+
+TEXT_DETECTION_CONFIDENCE_THRESHOLD = 0.75
\ No newline at end of file
