@@ -6,18 +6,30 @@
     If these aren't important to you, I highly recommend you use Sven Marnach's excellent 
     pyexiftool: https://github.com/smarnach/pyexiftool which provides more functionality """
 
+import atexit
 import json
 import logging
 import os
 import re
 import shutil
 import subprocess
+from abc import ABC, abstractmethod
 from functools import lru_cache  # pylint: disable=syntax-error
 
 # exiftool -stay_open commands outputs this EOF marker after command is run
 EXIFTOOL_STAYOPEN_EOF = "{ready}"
 EXIFTOOL_STAYOPEN_EOF_LEN = len(EXIFTOOL_STAYOPEN_EOF)
 
+# list of exiftool processes to cleanup when exiting or when terminate is called
+EXIFTOOL_PROCESSES = []
+
+
+@atexit.register
+def terminate_exiftool():
+    """Terminate any running ExifTool subprocesses; call this to cleanup when done using ExifTool """
+    for proc in EXIFTOOL_PROCESSES:
+        proc._stop_proc()
+
 
 @lru_cache(maxsize=1)
 def get_exiftool_path():
@@ -66,7 +78,8 @@ class _ExifToolProc:
         if self._process_running:
             return self._process
         else:
-            raise ValueError("exiftool process is not running")
+            self._start_proc()
+            return self._process
 
     @property
     def pid(self):
@@ -104,30 +117,30 @@ class _ExifToolProc:
         )
         self._process_running = True
 
+        EXIFTOOL_PROCESSES.append(self)
+
     def _stop_proc(self):
         """ stop the exiftool process if it's running, otherwise, do nothing """
 
         if not self._process_running:
             return
 
-        self._process.stdin.write(b"-stay_open\n")
-        self._process.stdin.write(b"False\n")
-        self._process.stdin.flush()
+        try:
+            self._process.stdin.write(b"-stay_open\n")
+            self._process.stdin.write(b"False\n")
+            self._process.stdin.flush()
+        except Exception as e:
+            pass
+
         try:
             self._process.communicate(timeout=5)
         except subprocess.TimeoutExpired:
-            logging.warning(
-                f"exiftool pid {self._process.pid} did not exit, killing it"
-            )
             self._process.kill()
             self._process.communicate()
 
         del self._process
         self._process_running = False
 
-    def __del__(self):
-        self._stop_proc()
-
 
 class ExifTool:
     """ Basic exiftool interface for reading and writing EXIF tags """
@@ -153,9 +166,12 @@ class ExifTool:
         # if running as a context manager, self._context_mgr will be True
         self._context_mgr = False
         self._exiftoolproc = _ExifToolProc(exiftool=exiftool)
-        self._process = self._exiftoolproc.process
         self._read_exif()
 
+    @property
+    def _process(self):
+        return self._exiftoolproc.process
+
     def setvalue(self, tag, value):
         """Set tag to value(s); if value is None, will delete tag
 
@@ -185,7 +201,7 @@ class ExifTool:
             return True
         else:
             _, _, error = self.run_commands(*command)
-            return error is None
+            return error == ""
 
     def addvalues(self, tag, *values):
         """Add one or more value(s) to tag
@@ -227,7 +243,7 @@ class ExifTool:
             return True
         else:
             _, _, error = self.run_commands(*command)
-            return error is None
+            return error == ""
 
     def run_commands(self, *commands, no_file=False):
         """Run commands in the exiftool process and return result.
@@ -308,12 +324,13 @@ class ExifTool:
         ver, _, _ = self.run_commands("-ver", no_file=True)
         return ver.decode("utf-8")
 
-    def asdict(self, tag_groups=True):
+    def asdict(self, tag_groups=True, normalized=False):
         """return dictionary of all EXIF tags and values from exiftool
         returns empty dict if no tags
 
         Args:
             tag_groups: if True (default), dict keys have tag groups, e.g. "IPTC:Keywords"; if False, drops groups from keys, e.g. "Keywords"
+            normalized: if True, dict keys are all normalized to lower case (default is False)
         """
         json_str, _, _ = self.run_commands("-json")
         if not json_str:
@@ -334,6 +351,10 @@ class ExifTool:
                 k = re.sub(r".*:", "", k)
                 exif_new[k] = v
             exifdict = exif_new
+
+        if normalized:
+            exifdict = {k.lower(): v for (k, v) in exifdict.items()}
+
         return exifdict
 
     def json(self):
@@ -360,3 +381,74 @@ class ExifTool:
         elif self._commands:
             # run_commands sets self.warning and self.error as needed
             self.run_commands(*self._commands)
+
+
+class ExifToolCaching(ExifTool):
+    """ Basic exiftool interface for reading and writing EXIF tags, with caching. 
+        Use this only when you know the file's EXIF data will not be changed by any external process. 
+        
+        Creates a singleton cached ExifTool instance """
+
+    _singletons = {}
+
+    def __new__(cls, filepath, exiftool=None):
+        """ create new object or return instance of already created singleton """
+        if filepath not in cls._singletons:
+            cls._singletons[filepath] = _ExifToolCaching(filepath, exiftool=exiftool)
+        return cls._singletons[filepath]
+
+
+class _ExifToolCaching(ExifTool):
+    def __init__(self, filepath, exiftool=None):
+        """Create read-only ExifTool object that caches values
+
+        Args:
+            file: path to image file
+            exiftool: path to exiftool, if not specified will look in path
+
+        Returns:
+            ExifTool instance
+        """
+        self._json_cache = None
+        self._asdict_cache = {}
+        super().__init__(filepath, exiftool=exiftool, overwrite=False, flags=None)
+
+    def run_commands(self, *commands, no_file=False):
+        if commands[0] not in ["-json", "-ver"]:
+            raise NotImplementedError(f"{self.__class__} is read-only")
+        return super().run_commands(*commands, no_file=no_file)
+
+    def setvalue(self, tag, value):
+        raise NotImplementedError(f"{self.__class__} is read-only")
+
+    def addvalues(self, tag, *values):
+        raise NotImplementedError(f"{self.__class__} is read-only")
+
+    def json(self):
+        if not self._json_cache:
+            self._json_cache = super().json()
+        return self._json_cache
+
+    def asdict(self, tag_groups=True, normalized=False):
+        """return dictionary of all EXIF tags and values from exiftool
+        returns empty dict if no tags
+
+        Args:
+            tag_groups: if True (default), dict keys have tag groups, e.g. "IPTC:Keywords"; if False, drops groups from keys, e.g. "Keywords"
+            normalized: if True, dict keys are all normalized to lower case (default is False)
+        """
+        try:
+            return self._asdict_cache[tag_groups][normalized]
+        except KeyError:
+            if tag_groups not in self._asdict_cache:
+                self._asdict_cache[tag_groups] = {}
+            self._asdict_cache[tag_groups][normalized] = super().asdict(
+                tag_groups=tag_groups, normalized=normalized
+            )
+            return self._asdict_cache[tag_groups][normalized]
+
+    def flush_cache(self):
+        """ Clear cached data so that calls to json or asdict return fresh data """
+        self._json_cache = None
+        self._asdict_cache = {}
+
