@@ -7,8 +7,11 @@ import subprocess
 import sys
 from abc import ABC, abstractmethod
 
+import CoreFoundation
+
 from .imageconverter import ImageConverter
 
+
 class FileUtilABC(ABC):
     """ Abstract base class for FileUtil """
 
@@ -27,6 +30,11 @@ class FileUtilABC(ABC):
     def unlink(cls, dest):
         pass
 
+    @classmethod
+    @abstractmethod
+    def rmdir(cls, dest):
+        pass
+
     @classmethod
     @abstractmethod
     def utime(cls, path, times):
@@ -52,6 +60,11 @@ class FileUtilABC(ABC):
     def convert_to_jpeg(cls, src_file, dest_file, compression_quality=1.0):
         pass
 
+    @classmethod
+    @abstractmethod
+    def rename(cls, src, dest):
+        pass
+
 
 class FileUtilMacOS(FileUtilABC):
     """ Various file utilities """
@@ -76,35 +89,38 @@ class FileUtilMacOS(FileUtilABC):
             raise e
 
     @classmethod
-    def copy(cls, src, dest, norsrc=False):
+    def copy(cls, src, dest):
         """ Copies a file from src path to dest path 
-            src: source path as string 
+        
+        Args:
+            src: source path as string; must be a valid file path
             dest: destination path as string
-            norsrc: (bool) if True, uses --norsrc flag with ditto so it will not copy
-                    resource fork or extended attributes.  May be useful on volumes that
-                    don't work with extended attributes (likely only certain SMB mounts)
-                    default is False
-            Uses ditto to perform copy; will silently overwrite dest if it exists
-            Raises exception if copy fails or either path is None """
+                  dest may be either directory or file; in either case, src file must not exist in dest
+            Note: src and dest may be either a string or a pathlib.Path object
+        
+        Returns:
+            True if copy succeeded
+        
+        Raises:
+            OSError if copy fails
+            TypeError if either path is None
+        """
+        if not isinstance(src, pathlib.Path):
+            src = pathlib.Path(src)
 
-        if src is None or dest is None:
-            raise ValueError("src and dest must not be None", src, dest)
+        if not isinstance(dest, pathlib.Path):
+            dest = pathlib.Path(dest)
 
-        if not os.path.exists(src):
-            raise FileNotFoundError("src file does not appear to exist", src)
+        if dest.is_dir():
+            dest /= src.name
 
-        if norsrc:
-            command = ["/usr/bin/ditto", "--norsrc", src, dest]
-        else:
-            command = ["/usr/bin/ditto", src, dest]
-
-        # if error on copy, subprocess will raise CalledProcessError
-        try:
-            result = subprocess.run(command, check=True, stderr=subprocess.PIPE)
-        except subprocess.CalledProcessError as e:
-            raise e
-
-        return result.returncode
+        filemgr = CoreFoundation.NSFileManager.defaultManager()
+        error = filemgr.copyItemAtPath_toPath_error_(str(src), str(dest), None)
+        # error is a tuple of (bool, error_string)
+        # error[0] is True if copy succeeded
+        if not error[0]:
+            raise OSError(error[1])
+        return True
 
     @classmethod
     def unlink(cls, filepath):
@@ -114,6 +130,14 @@ class FileUtilMacOS(FileUtilABC):
         else:
             os.unlink(filepath)
 
+    @classmethod
+    def rmdir(cls, dirpath):
+        """ remove directory filepath; dirpath must be empty """
+        if isinstance(dirpath, pathlib.Path):
+            dirpath.rmdir()
+        else:
+            os.rmdir(dirpath)
+
     @classmethod
     def utime(cls, path, times):
         """ Set the access and modified time of path. """
@@ -164,7 +188,7 @@ class FileUtilMacOS(FileUtilABC):
     def file_sig(cls, f1):
         """ return os.stat signature for file f1 """
         return cls._sig(os.stat(f1))
-    
+
     @classmethod
     def convert_to_jpeg(cls, src_file, dest_file, compression_quality=1.0):
         """ converts image file src_file to jpeg format as dest_file
@@ -178,7 +202,24 @@ class FileUtilMacOS(FileUtilABC):
                 True if success, otherwise False 
         """
         converter = ImageConverter()
-        return converter.write_jpeg(src_file, dest_file, compression_quality=compression_quality)
+        return converter.write_jpeg(
+            src_file, dest_file, compression_quality=compression_quality
+        )
+
+    @classmethod
+    def rename(cls, src, dest):
+        """ Copy src to dest 
+
+        Args:
+            src: path to source file
+            dest: path to destination file
+        
+        Returns:
+            Name of renamed file (dest)
+        
+        """
+        os.rename(str(src), str(dest))
+        return dest
 
     @staticmethod
     def _sig(st):
@@ -189,6 +230,7 @@ class FileUtilMacOS(FileUtilABC):
         # use int(st.st_mtime) because ditto does not copy fractional portion of mtime
         return (stat.S_IFMT(st.st_mode), st.st_size, int(st.st_mtime))
 
+
 class FileUtil(FileUtilMacOS):
     """ Various file utilities """
 
@@ -228,6 +270,10 @@ class FileUtilNoOp(FileUtil):
     def unlink(cls, dest):
         cls.verbose(f"unlink: {dest}")
 
+    @classmethod
+    def rmdir(cls, dest):
+        cls.verbose(f"rmdir: {dest}")
+
     @classmethod
     def utime(cls, path, times):
         cls.verbose(f"utime: {path}, {times}")
@@ -240,3 +286,7 @@ class FileUtilNoOp(FileUtil):
     @classmethod
     def convert_to_jpeg(cls, src_file, dest_file, compression_quality=1.0):
         cls.verbose(f"convert_to_jpeg: {src_file}, {dest_file}, {compression_quality}")
+
+    @classmethod
+    def rename(cls, src, dest):
+        cls.verbose(f"rename: {src}, {dest}")
