@@ -3,7 +3,6 @@ PhotoInfo class
 Represents a single photo in the Photos library and provides access to the photo's attributes
 PhotosDB.photos() returns a list of PhotoInfo objects
 """
-
 import dataclasses
 import datetime
 import json
@@ -12,6 +11,7 @@ import os
 import os.path
 import pathlib
 from datetime import timedelta, timezone
+from typing import Optional
 
 import yaml
 
@@ -26,13 +26,20 @@ from .._constants import (
     _PHOTOS_5_SHARED_ALBUM_KIND,
     _PHOTOS_5_SHARED_PHOTO_PATH,
     _PHOTOS_5_VERSION,
+    BURST_DEFAULT_PICK,
+    BURST_KEY,
+    BURST_NOT_SELECTED,
+    BURST_SELECTED,
+    TEXT_DETECTION_CONFIDENCE_THRESHOLD,
 )
 from ..adjustmentsinfo import AdjustmentsInfo
 from ..albuminfo import AlbumInfo, ImportInfo
 from ..personinfo import FaceInfo, PersonInfo
-from ..phototemplate import PhotoTemplate
+from ..phototemplate import PhotoTemplate, RenderOptions
 from ..placeinfo import PlaceInfo4, PlaceInfo5
-from ..utils import _debug, _get_resource_loc, findfiles, get_preferred_uti_extension
+from ..text_detection import detect_text
+from ..uti import get_preferred_uti_extension, get_uti_for_extension
+from ..utils import _debug, _get_resource_loc, findfiles
 
 
 class PhotoInfo:
@@ -42,30 +49,31 @@ class PhotoInfo:
     """
 
     # import additional methods
-    from ._photoinfo_searchinfo import (
-        search_info,
-        search_info_normalized,
-        labels,
-        labels_normalized,
-        SearchInfo,
-    )
-    from ._photoinfo_exifinfo import exif_info, ExifInfo
+    from ._photoinfo_comments import comments, likes
+    from ._photoinfo_exifinfo import ExifInfo, exif_info
     from ._photoinfo_exiftool import exiftool
     from ._photoinfo_export import (
-        export,
-        export2,
-        _export_photo,
+        ExportResults,
         _exiftool_dict,
         _exiftool_json_sidecar,
+        _export_photo,
+        _export_photo_with_photos_export,
         _get_exif_keywords,
         _get_exif_persons,
         _write_exif_data,
         _write_sidecar,
         _xmp_sidecar,
-        ExportResults,
+        export,
+        export2,
+    )
+    from ._photoinfo_scoreinfo import ScoreInfo, score
+    from ._photoinfo_searchinfo import (
+        SearchInfo,
+        labels,
+        labels_normalized,
+        search_info,
+        search_info_normalized,
     )
-    from ._photoinfo_scoreinfo import score, ScoreInfo
-    from ._photoinfo_comments import comments, likes
 
     def __init__(self, db=None, uuid=None, info=None):
         self._uuid = uuid
@@ -73,9 +81,12 @@ class PhotoInfo:
         self._db = db
         self._verbose = self._db._verbose
 
+        # TODO: remove this once refactor of PhotoExporter is done
+        self._render_options = RenderOptions()
+
     @property
     def filename(self):
-        """ filename of the picture """
+        """filename of the picture"""
         if (
             self._db._db_version <= _PHOTOS_4_VERSION
             and self.has_raw
@@ -103,7 +114,7 @@ class PhotoInfo:
 
     @property
     def date(self):
-        """ image creation date as timezone aware datetime object """
+        """image creation date as timezone aware datetime object"""
         return self._info["imageDate"]
 
     @property
@@ -129,52 +140,22 @@ class PhotoInfo:
 
     @property
     def tzoffset(self):
-        """ timezone offset from UTC in seconds """
+        """timezone offset from UTC in seconds"""
         return self._info["imageTimeZoneOffsetSeconds"]
 
     @property
     def path(self):
-        """ absolute path on disk of the original picture """
+        """absolute path on disk of the original picture"""
         try:
             return self._path
         except AttributeError:
             self._path = None
             photopath = None
-            # TODO: should path try to return path even if ismissing?
             if self._info["isMissing"] == 1:
                 return photopath  # path would be meaningless until downloaded
 
             if self._db._db_version <= _PHOTOS_4_VERSION:
-                if self._info["has_raw"]:
-                    # return the path to JPEG even if RAW is original
-                    vol = (
-                        self._db._dbvolumes[self._info["raw_pair_info"]["volumeId"]]
-                        if self._info["raw_pair_info"]["volumeId"] is not None
-                        else None
-                    )
-                    if vol is not None:
-                        photopath = os.path.join(
-                            "/Volumes", vol, self._info["raw_pair_info"]["imagePath"]
-                        )
-                    else:
-                        photopath = os.path.join(
-                            self._db._masters_path,
-                            self._info["raw_pair_info"]["imagePath"],
-                        )
-                else:
-                    vol = self._info["volume"]
-                    if vol is not None:
-                        photopath = os.path.join(
-                            "/Volumes", vol, self._info["imagePath"]
-                        )
-                    else:
-                        photopath = os.path.join(
-                            self._db._masters_path, self._info["imagePath"]
-                        )
-                if not os.path.isfile(photopath):
-                    photopath = None
-                self._path = photopath
-                return photopath
+                return self._path_4()
 
             if self._info["shared"]:
                 # shared photo
@@ -204,9 +185,40 @@ class PhotoInfo:
             self._path = photopath
             return photopath
 
+    def _path_4(self):
+        """return path for photo on Photos <= version 4"""
+        if self._info["has_raw"]:
+            # return the path to JPEG even if RAW is original
+            vol = (
+                self._db._dbvolumes[self._info["raw_pair_info"]["volumeId"]]
+                if self._info["raw_pair_info"]["volumeId"] is not None
+                else None
+            )
+            if vol is not None:
+                photopath = os.path.join(
+                    "/Volumes", vol, self._info["raw_pair_info"]["imagePath"]
+                )
+            else:
+                photopath = os.path.join(
+                    self._db._masters_path,
+                    self._info["raw_pair_info"]["imagePath"],
+                )
+        else:
+            vol = self._info["volume"]
+            if vol is not None:
+                photopath = os.path.join("/Volumes", vol, self._info["imagePath"])
+            else:
+                photopath = os.path.join(
+                    self._db._masters_path, self._info["imagePath"]
+                )
+        if not os.path.isfile(photopath):
+            photopath = None
+        self._path = photopath
+        return photopath
+
     @property
     def path_edited(self):
-        """ absolute path on disk of the edited picture """
+        """absolute path on disk of the edited picture"""
         """ None if photo has not been edited """
 
         try:
@@ -220,7 +232,7 @@ class PhotoInfo:
             return self._path_edited
 
     def _path_edited_5(self):
-        """ return path_edited for Photos >= 5 """
+        """return path_edited for Photos >= 5"""
         # In Photos 5.0 / Catalina / MacOS 10.15:
         # edited photos appear to always be converted to .jpeg and stored in
         # library_name/resources/renders/X/UUID_1_201_a.jpeg
@@ -243,14 +255,10 @@ class PhotoInfo:
             filename = None
             if self._info["type"] == _PHOTO_TYPE:
                 # it's a photo
-                if self._db._photos_ver == 5:
-                    filename = f"{self._uuid}_1_201_a.jpeg"
+                if self._db._photos_ver != 5 and self.uti == "public.heic":
+                    filename = f"{self._uuid}_1_201_a.heic"
                 else:
-                    # could be a heic or a jpeg
-                    if self.uti == "public.heic":
-                        filename = f"{self._uuid}_1_201_a.heic"
-                    else:
-                        filename = f"{self._uuid}_1_201_a.jpeg"
+                    filename = f"{self._uuid}_1_201_a.jpeg"
             elif self._info["type"] == _MOVIE_TYPE:
                 # it's a movie
                 filename = f"{self._uuid}_2_0_a.mov"
@@ -278,7 +286,7 @@ class PhotoInfo:
         return photopath
 
     def _path_edited_4(self):
-        """ return path_edited for Photos <= 4 """
+        """return path_edited for Photos <= 4"""
 
         if self._db._db_version > _PHOTOS_4_VERSION:
             raise RuntimeError("Wrong database format!")
@@ -336,9 +344,40 @@ class PhotoInfo:
 
         return photopath
 
+    @property
+    def path_edited_live_photo(self):
+        """return path to edited version of live photo movie; only valid for Photos 5+"""
+        if self._db._db_version < _PHOTOS_5_VERSION:
+            return None
+
+        try:
+            return self._path_edited_live_photo
+        except AttributeError:
+            self._path_edited_live_photo = self._path_edited_5_live_photo()
+            return self._path_edited_live_photo
+
+    def _path_edited_5_live_photo(self):
+        """return path_edited_live_photo for Photos >= 5"""
+        if self._db._db_version < _PHOTOS_5_VERSION:
+            raise RuntimeError("Wrong database format!")
+
+        if self.live_photo and self._info["hasAdjustments"]:
+            library = self._db._library_path
+            directory = self._uuid[0]  # first char of uuid
+            filename = f"{self._uuid}_2_100_a.mov"
+            photopath = os.path.join(
+                library, "resources", "renders", directory, filename
+            )
+            if not os.path.isfile(photopath):
+                photopath = None
+        else:
+            photopath = None
+
+        return photopath
+
     @property
     def path_raw(self):
-        """ absolute path of associated RAW image or None if there is not one """
+        """absolute path of associated RAW image or None if there is not one"""
 
         # In Photos 5, raw is in same folder as original but with _4.ext
         # Unless "Copy Items to the Photos Library" is not checked
@@ -365,60 +404,72 @@ class PhotoInfo:
         #     return photopath
 
         if self._db._db_version <= _PHOTOS_4_VERSION:
-            vol = self._info["raw_info"]["volume"]
-            if vol is not None:
-                photopath = os.path.join(
-                    "/Volumes", vol, self._info["raw_info"]["imagePath"]
-                )
-            else:
-                photopath = os.path.join(
-                    self._db._masters_path, self._info["raw_info"]["imagePath"]
-                )
-            if not os.path.isfile(photopath):
-                logging.debug(
-                    f"MISSING PATH: RAW photo for UUID {self._uuid} should be at {photopath} but does not appear to exist"
-                )
-                photopath = None
-        else:
+            return self._path_raw_4()
+
+        if not self.isreference:
             filestem = pathlib.Path(self._info["filename"]).stem
-            raw_ext = get_preferred_uti_extension(self._info["UTI_raw"])
+            # raw_ext = get_preferred_uti_extension(self._info["UTI_raw"])
 
             if self._info["directory"].startswith("/"):
                 filepath = self._info["directory"]
             else:
                 filepath = os.path.join(self._db._masters_path, self._info["directory"])
 
-            glob_str = f"{filestem}*.{raw_ext}"
+            # raw files have same name as original but with _4.raw_ext appended
+            # I believe the _4 maps to PHAssetResourceTypeAlternatePhoto = 4
+            # see: https://developer.apple.com/documentation/photokit/phassetresourcetype/phassetresourcetypealternatephoto?language=objc
+            glob_str = f"{filestem}_4*"
             raw_file = findfiles(glob_str, filepath)
-            if len(raw_file) != 1:
-                # Note: In Photos Version 5.0 (141.19.150), images not copied to Photos Library
-                # that are missing do not always trigger is_missing = True as happens
-                # in earlier version so it's possible for this check to fail, if so, return None
-                logging.debug(f"Error getting path to RAW file: {filepath}/{glob_str}")
+            if not raw_file:
                 photopath = None
             else:
-                photopath = os.path.join(filepath, raw_file[0])
-                if not os.path.isfile(photopath):
-                    logging.debug(
-                        f"MISSING PATH: RAW photo for UUID {self._uuid} should be at {photopath} but does not appear to exist"
-                    )
-                    photopath = None
+                photopath = pathlib.Path(filepath) / raw_file[0]
+                photopath = str(photopath) if photopath.is_file() else None
+        else:
+            # is a reference
+            try:
+                photopath = (
+                    pathlib.Path("/Volumes")
+                    / self._info["raw_volume"]
+                    / self._info["raw_relative_path"]
+                )
+                photopath = str(photopath) if photopath.is_file() else None
+            except KeyError:
+                # don't have the path details
+                photopath = None
 
         return photopath
 
+    def _path_raw_4(self):
+        """Return path_raw for Photos <= version 4"""
+        vol = self._info["raw_info"]["volume"]
+        if vol is not None:
+            photopath = os.path.join(
+                "/Volumes", vol, self._info["raw_info"]["imagePath"]
+            )
+        else:
+            photopath = os.path.join(
+                self._db._masters_path, self._info["raw_info"]["imagePath"]
+            )
+        if not os.path.isfile(photopath):
+            logging.debug(
+                f"MISSING PATH: RAW photo for UUID {self._uuid} should be at {photopath} but does not appear to exist"
+            )
+            photopath = None
+
     @property
     def description(self):
-        """ long / extended description of picture """
+        """long / extended description of picture"""
         return self._info["extendedDescription"]
 
     @property
     def persons(self):
-        """ list of persons in picture """
+        """list of persons in picture"""
         return [self._db._dbpersons_pk[pk]["fullname"] for pk in self._info["persons"]]
 
     @property
     def person_info(self):
-        """ list of PersonInfo objects for person in picture """
+        """list of PersonInfo objects for person in picture"""
         try:
             return self._personinfo
         except AttributeError:
@@ -429,7 +480,7 @@ class PhotoInfo:
 
     @property
     def face_info(self):
-        """ list of FaceInfo objects for faces in picture """
+        """list of FaceInfo objects for faces in picture"""
         try:
             return self._faceinfo
         except AttributeError:
@@ -443,7 +494,7 @@ class PhotoInfo:
 
     @property
     def albums(self):
-        """ list of albums picture is contained in """
+        """list of albums picture is contained in"""
         try:
             return self._albums
         except AttributeError:
@@ -455,22 +506,20 @@ class PhotoInfo:
 
     @property
     def burst_albums(self):
-        """If photo is non-selected burst photo, list of albums any other images in the same burst set are contained in, otherwise returns self.albums"""
-        if self.burst_selected or not self.burst:
-            return self.albums
-
+        """If photo is burst photo, list of albums it is contained in as well as any albums the key photo is contained in, otherwise returns self.albums"""
         try:
             return self._burst_albums
         except AttributeError:
-            burst_albums = []
+            burst_albums = list(self.albums)
             for photo in self.burst_photos:
-                burst_albums.extend(photo.albums)
+                if photo.burst_key:
+                    burst_albums.extend(photo.albums)
             self._burst_albums = list(set(burst_albums))
             return self._burst_albums
 
     @property
     def album_info(self):
-        """ list of AlbumInfo objects representing albums the photo is contained in """
+        """list of AlbumInfo objects representing albums the photo is contained in"""
         try:
             return self._album_info
         except AttributeError:
@@ -482,22 +531,20 @@ class PhotoInfo:
 
     @property
     def burst_album_info(self):
-        """ If photo is a non-selected burst photo, returns list of AlbumInfo objects representing albums any other photos in the same burst set are contained in, otherwise returns self.album_info """
-        if self.burst_selected or not self.burst:
-            return self.album_info
-
+        """If photo is a burst photo, returns list of AlbumInfo objects representing albums the photo is contained in as well as albums the burst key photo is contained in, otherwise returns self.album_info."""
         try:
             return self._burst_album_info
         except AttributeError:
-            burst_album_info = []
+            burst_album_info = list(self.album_info)
             for photo in self.burst_photos:
-                burst_album_info.extend(photo.album_info)
+                if photo.burst_key:
+                    burst_album_info.extend(photo.album_info)
             self._burst_album_info = list(set(burst_album_info))
             return self._burst_album_info
 
     @property
     def import_info(self):
-        """ ImportInfo object representing import session for the photo or None if no import session """
+        """ImportInfo object representing import session for the photo or None if no import session"""
         try:
             return self._import_info
         except AttributeError:
@@ -510,17 +557,17 @@ class PhotoInfo:
 
     @property
     def keywords(self):
-        """ list of keywords for picture """
+        """list of keywords for picture"""
         return self._info["keywords"]
 
     @property
     def title(self):
-        """ name / title of picture """
+        """name / title of picture"""
         return self._info["name"]
 
     @property
     def uuid(self):
-        """ UUID of picture """
+        """UUID of picture"""
         return self._uuid
 
     @property
@@ -538,12 +585,12 @@ class PhotoInfo:
 
     @property
     def hasadjustments(self):
-        """ True if picture has adjustments / edits """
+        """True if picture has adjustments / edits"""
         return self._info["hasAdjustments"] == 1
 
     @property
     def adjustments(self):
-        """ Returns AdjustmentsInfo class for adjustment data or None if no adjustments; Photos 5+ only """
+        """Returns AdjustmentsInfo class for adjustment data or None if no adjustments; Photos 5+ only"""
         if self._db._db_version <= _PHOTOS_4_VERSION:
             return None
 
@@ -567,32 +614,32 @@ class PhotoInfo:
 
     @property
     def external_edit(self):
-        """ Returns True if picture was edited outside of Photos using external editor """
+        """Returns True if picture was edited outside of Photos using external editor"""
         return self._info["adjustmentFormatID"] == "com.apple.Photos.externalEdit"
 
     @property
     def favorite(self):
-        """ True if picture is marked as favorite """
+        """True if picture is marked as favorite"""
         return self._info["favorite"] == 1
 
     @property
     def hidden(self):
-        """ True if picture is hidden """
+        """True if picture is hidden"""
         return self._info["hidden"] == 1
 
     @property
     def visible(self):
-        """ True if picture is visble """
+        """True if picture is visble"""
         return self._info["visible"]
 
     @property
     def intrash(self):
-        """ True if picture is in trash ('Recently Deleted' folder)"""
+        """True if picture is in trash ('Recently Deleted' folder)"""
         return self._info["intrash"]
 
     @property
     def date_trashed(self):
-        """ Date asset was placed in the trash or None """
+        """Date asset was placed in the trash or None"""
         # TODO: add add_timezone(dt, offset_seconds) to datetime_utils
         # also update date_modified
         trasheddate = self._info["trasheddate"]
@@ -604,9 +651,26 @@ class PhotoInfo:
         else:
             return None
 
+    @property
+    def date_added(self):
+        """Date photo was added to the database"""
+        try:
+            return self._date_added
+        except AttributeError:
+            added_date = self._info["added_date"]
+            if added_date:
+                seconds = self._info["imageTimeZoneOffsetSeconds"] or 0
+                delta = timedelta(seconds=seconds)
+                tz = timezone(delta)
+                self._date_added = added_date.astimezone(tz=tz)
+            else:
+                self._date_added = None
+
+            return self._date_added
+
     @property
     def location(self):
-        """ returns (latitude, longitude) as float in degrees or None """
+        """returns (latitude, longitude) as float in degrees or None"""
         return (self._latitude, self._longitude)
 
     @property
@@ -640,13 +704,23 @@ class PhotoInfo:
         """Returns Uniform Type Identifier (UTI) for the original image
         for example: public.jpeg or com.apple.quicktime-movie
         """
-        if self._db._db_version <= _PHOTOS_4_VERSION and self._info["has_raw"]:
-            return self._info["raw_pair_info"]["UTI"]
-        elif self.shared:
-            # TODO: need reliable way to get original UTI for shared
-            return self.uti
-        else:
-            return self._info["UTI_original"]
+        try:
+            return self._uti_original
+        except AttributeError:
+            if self._db._db_version <= _PHOTOS_4_VERSION and self._info["has_raw"]:
+                self._uti_original = self._info["raw_pair_info"]["UTI"]
+            elif self.shared:
+                # TODO: need reliable way to get original UTI for shared
+                self._uti_original = self.uti
+            elif self._db._photos_ver >= 7:
+                # Monterey+
+                self._uti_original = get_uti_for_extension(
+                    pathlib.Path(self.original_filename).suffix
+                )
+            else:
+                self._uti_original = self._info["UTI_original"]
+
+            return self._uti_original
 
     @property
     def uti_edited(self):
@@ -665,7 +739,14 @@ class PhotoInfo:
         for example: com.canon.cr2-raw-image
         Returns None if no associated RAW image
         """
-        return self._info["UTI_raw"]
+        if self._db._photos_ver < 7:
+            return self._info["UTI_raw"]
+
+        rawpath = self.path_raw
+        if rawpath:
+            return get_uti_for_extension(pathlib.Path(rawpath).suffix)
+        else:
+            return None
 
     @property
     def ismovie(self):
@@ -702,18 +783,28 @@ class PhotoInfo:
 
     @property
     def isreference(self):
-        """ Returns True if photo is a reference (not copied to the Photos library), otherwise False """
+        """Returns True if photo is a reference (not copied to the Photos library), otherwise False"""
         return self._info["isreference"]
 
     @property
     def burst(self):
-        """ Returns True if photo is part of a Burst photo set, otherwise False """
+        """Returns True if photo is part of a Burst photo set, otherwise False"""
         return self._info["burst"]
 
     @property
     def burst_selected(self):
-        """ Returns True if photo is a burst photo and has been selected from the burst set by the user, otherwise False """
-        return self._info["burst_key"]
+        """Returns True if photo is a burst photo and has been selected from the burst set by the user, otherwise False"""
+        return bool(self._info["burstPickType"] & BURST_SELECTED)
+
+    @property
+    def burst_key(self):
+        """Returns True if photo is a burst photo and is the key image for the burst set (the image that Photos shows on top of the burst stack), otherwise False"""
+        return bool(self._info["burstPickType"] & BURST_KEY)
+
+    @property
+    def burst_default_pick(self):
+        """Returns True if photo is a burst image and is the photo that Photos selected as the default image for the burst set, otherwise False"""
+        return bool(self._info["burstPickType"] & BURST_DEFAULT_PICK)
 
     @property
     def burst_photos(self):
@@ -732,7 +823,7 @@ class PhotoInfo:
 
     @property
     def live_photo(self):
-        """ Returns True if photo is a live photo, otherwise False """
+        """Returns True if photo is a live photo, otherwise False"""
         return self._info["live_photo"]
 
     @property
@@ -791,44 +882,113 @@ class PhotoInfo:
 
         return photopath
 
+    @property
+    def path_derivatives(self):
+        """Return any derivative (preview) images associated with the photo as a list of paths, sorted by file size (largest first)"""
+        try:
+            return self._path_derivatives
+        except AttributeError:
+            if self._db._db_version <= _PHOTOS_4_VERSION:
+                self._path_derivatives = self._path_derivatives_4()
+                return self._path_derivatives
+
+            directory = self._uuid[0]  # first char of uuid
+            derivative_path = (
+                pathlib.Path(self._db._library_path)
+                / "resources"
+                / "derivatives"
+                / directory
+            )
+            files = derivative_path.glob(f"{self.uuid}*.*")
+            files = sorted(files, reverse=True, key=lambda f: f.stat().st_size)
+            # return list of filename but skip .THM files (these are actually low-res thumbnails in JPEG format but with .THM extension)
+            derivatives = [
+                str(filename) for filename in files if filename.suffix != ".THM"
+            ]
+            if (
+                self.isphoto
+                and len(derivatives) > 1
+                and derivatives[0].endswith(".mov")
+            ):
+                derivatives[1], derivatives[0] = derivatives[0], derivatives[1]
+
+            self._path_derivatives = derivatives
+            return self._path_derivatives
+
+    def _path_derivatives_4(self):
+        """Return paths to all derivative (preview) files for Photos <= 4"""
+        modelid = self._info["modelID"]
+        if modelid is None:
+            return []
+        folder_id, file_id = _get_resource_loc(modelid)
+        derivatives_root = (
+            pathlib.Path(self._db._library_path)
+            / "resources"
+            / "proxies"
+            / "derivatives"
+            / folder_id
+        )
+
+        # photos appears to usually be in "00" subfolder but
+        # could be elsewhere--I haven't figured out this logic yet
+        # first see if it's in 00
+
+        derivatives_path = derivatives_root / "00" / file_id
+        if derivatives_path.is_dir():
+            files = derivatives_path.glob("*")
+            files = sorted(files, reverse=True, key=lambda f: f.stat().st_size)
+            return [str(filename) for filename in files]
+
+        # didn't find derivatives path
+        for subdir in derivatives_root.glob("*"):
+            if subdir.is_dir():
+                derivatives_path = derivatives_root / subdir / file_id
+                if derivatives_path.is_dir():
+                    files = derivatives_path.glob("*")
+                    files = sorted(files, reverse=True, key=lambda f: f.stat().st_size)
+                    return [str(filename) for filename in files]
+
+        # didn't find a derivatives path
+        return []
+
     @property
     def panorama(self):
-        """ Returns True if photo is a panorama, otherwise False """
+        """Returns True if photo is a panorama, otherwise False"""
         return self._info["panorama"]
 
     @property
     def slow_mo(self):
-        """ Returns True if photo is a slow motion video, otherwise False """
+        """Returns True if photo is a slow motion video, otherwise False"""
         return self._info["slow_mo"]
 
     @property
     def time_lapse(self):
-        """ Returns True if photo is a time lapse video, otherwise False """
+        """Returns True if photo is a time lapse video, otherwise False"""
         return self._info["time_lapse"]
 
     @property
     def hdr(self):
-        """ Returns True if photo is an HDR photo, otherwise False """
+        """Returns True if photo is an HDR photo, otherwise False"""
         return self._info["hdr"]
 
     @property
     def screenshot(self):
-        """ Returns True if photo is an HDR photo, otherwise False """
+        """Returns True if photo is an HDR photo, otherwise False"""
         return self._info["screenshot"]
 
     @property
     def portrait(self):
-        """ Returns True if photo is a portrait, otherwise False """
+        """Returns True if photo is a portrait, otherwise False"""
         return self._info["portrait"]
 
     @property
     def selfie(self):
-        """ Returns True if photo is a selfie (front facing camera), otherwise False """
+        """Returns True if photo is a selfie (front facing camera), otherwise False"""
         return self._info["selfie"]
 
     @property
     def place(self):
-        """ Returns PlaceInfo object containing reverse geolocation info """
+        """Returns PlaceInfo object containing reverse geolocation info"""
 
         # implementation note: doesn't create the PlaceInfo object until requested
         # then memoizes the object in self._place to avoid recreating the object
@@ -856,12 +1016,12 @@ class PhotoInfo:
 
     @property
     def has_raw(self):
-        """ returns True if photo has an associated raw image (that is, it's a RAW+JPEG pair), otherwise False """
+        """returns True if photo has an associated raw image (that is, it's a RAW+JPEG pair), otherwise False"""
         return self._info["has_raw"]
 
     @property
     def israw(self):
-        """ returns True if photo is a raw image. For images with an associated RAW+JPEG pair, see has_raw """
+        """returns True if photo is a raw image. For images with an associated RAW+JPEG pair, see has_raw"""
         return "raw-image" in self.uti_original
 
     @property
@@ -873,17 +1033,17 @@ class PhotoInfo:
 
     @property
     def height(self):
-        """ returns height of the current photo version in pixels """
+        """returns height of the current photo version in pixels"""
         return self._info["height"]
 
     @property
     def width(self):
-        """ returns width of the current photo version in pixels """
+        """returns width of the current photo version in pixels"""
         return self._info["width"]
 
     @property
     def orientation(self):
-        """ returns EXIF orientation of the current photo version as int or 0 if current orientation cannot be determined """
+        """returns EXIF orientation of the current photo version as int or 0 if current orientation cannot be determined"""
         if self._db._db_version <= _PHOTOS_4_VERSION:
             return self._info["orientation"]
 
@@ -899,73 +1059,98 @@ class PhotoInfo:
 
     @property
     def original_height(self):
-        """ returns height of the original photo version in pixels """
+        """returns height of the original photo version in pixels"""
         return self._info["original_height"]
 
     @property
     def original_width(self):
-        """ returns width of the original photo version in pixels """
+        """returns width of the original photo version in pixels"""
         return self._info["original_width"]
 
     @property
     def original_orientation(self):
-        """ returns EXIF orientation of the original photo version as int """
+        """returns EXIF orientation of the original photo version as int"""
         return self._info["original_orientation"]
 
     @property
     def original_filesize(self):
-        """ returns filesize of original photo in bytes as int """
+        """returns filesize of original photo in bytes as int"""
         return self._info["original_filesize"]
 
+    @property
+    def duplicates(self):
+        """return list of PhotoInfo objects for possible duplicates (matching signature of original size, date, height, width) or empty list if no matching duplicates"""
+        signature = self._db._duplicate_signature(self.uuid)
+        duplicates = []
+        try:
+            for uuid in self._db._db_signatures[signature]:
+                if uuid != self.uuid:
+                    # found a possible duplicate
+                    duplicates.append(self._db.get_photo(uuid))
+        except KeyError:
+            # don't expect this to happen as the signature should be in db
+            logging.warning(f"Did not find signature for {self.uuid} in _db_signatures")
+        return duplicates
+
     def render_template(
-        self,
-        template_str,
-        none_str="_",
-        path_sep=None,
-        expand_inplace=False,
-        inplace_sep=None,
-        filename=False,
-        dirname=False,
-        strip=False,
+        self, template_str: str, options: Optional[RenderOptions] = None
     ):
         """Renders a template string for PhotoInfo instance using PhotoTemplate
 
         Args:
             template_str: a template string with fields to render
-            none_str: a str to use if template field renders to None, default is "_".
-            path_sep: a single character str to use as path separator when joining
-                fields like folder_album; if not provided, defaults to os.path.sep
-            expand_inplace: expand multi-valued substitutions in-place as a single string
-                instead of returning individual strings
-            inplace_sep: optional string to use as separator between multi-valued keywords
-                with expand_inplace; default is ','
-            filename: if True, template output will be sanitized to produce valid file name
-            dirname: if True, template output will be sanitized to produce valid directory name
-            strip: if True, strips leading/trailing white space from resulting template
+            options: a RenderOptions instance
 
         Returns:
             ([rendered_strings], [unmatched]): tuple of list of rendered strings and list of unmatched template values
         """
+        options = options or RenderOptions()
         template = PhotoTemplate(self, exiftool_path=self._db._exiftool_path)
-        return template.render(
-            template_str,
-            none_str=none_str,
-            path_sep=path_sep,
-            expand_inplace=expand_inplace,
-            inplace_sep=inplace_sep,
-            filename=filename,
-            dirname=dirname,
-            strip=strip,
+        return template.render(template_str, options)
+
+    def detected_text(self, confidence_threshold=TEXT_DETECTION_CONFIDENCE_THRESHOLD):
+        """Detects text in photo and returns lists of results as (detected text, confidence)
+
+        confidence_threshold: float between 0.0 and 1.0. If text detection confidence is below this threshold,
+        text will not be returned. Default is TEXT_DETECTION_CONFIDENCE_THRESHOLD
+
+        If photo is edited, uses the edited photo, otherwise the original; falls back to the preview image if neither edited or original is available
+
+        Returns: list of (detected text, confidence) tuples
+        """
+        path = (
+            self.path_edited if self.hasadjustments and self.path_edited else self.path
         )
+        path = path or self.path_derivatives[0] if self.path_derivatives else None
+        if not path:
+            return []
+
+        try:
+            return self._detected_text[(path, confidence_threshold)]
+        except (AttributeError, KeyError) as e:
+            if isinstance(e, AttributeError):
+                self._detected_text = {}
+
+            try:
+                detected_text = detect_text(path)
+            except Exception as e:
+                detected_text = []
+
+            self._detected_text[(path, confidence_threshold)] = [
+                (text, confidence)
+                for text, confidence in detected_text
+                if confidence >= confidence_threshold
+            ]
+            return self._detected_text[(path, confidence_threshold)]
 
     @property
     def _longitude(self):
-        """ Returns longitude, in degrees """
+        """Returns longitude, in degrees"""
         return self._info["longitude"]
 
     @property
     def _latitude(self):
-        """ Returns latitude, in degrees """
+        """Returns latitude, in degrees"""
         return self._info["latitude"]
 
     def _get_album_uuids(self):
@@ -1003,7 +1188,7 @@ class PhotoInfo:
         return f"osxphotos.{self.__class__.__name__}(db={self._db}, uuid='{self._uuid}', info={self._info})"
 
     def __str__(self):
-        """ string representation of PhotoInfo object """
+        """string representation of PhotoInfo object"""
 
         date_iso = self.date.isoformat()
         date_modified_iso = (
@@ -1066,7 +1251,7 @@ class PhotoInfo:
         return yaml.dump(info, sort_keys=False)
 
     def asdict(self):
-        """ return dict representation """
+        """return dict representation"""
 
         folders = {album.title: album.folder_names for album in self.album_info}
         exif = dataclasses.asdict(self.exif_info) if self.exif_info else {}
@@ -1142,7 +1327,7 @@ class PhotoInfo:
         }
 
     def json(self):
-        """ Return JSON representation """
+        """Return JSON representation"""
 
         def default(o):
             if isinstance(o, (datetime.date, datetime.datetime)):
@@ -1151,7 +1336,7 @@ class PhotoInfo:
         return json.dumps(self.asdict(), sort_keys=True, default=default)
 
     def __eq__(self, other):
-        """ Compare two PhotoInfo objects for equality """
+        """Compare two PhotoInfo objects for equality"""
         # Can't just compare the two __dicts__ because some methods (like albums)
         # memoize their value once called in an instance variable (e.g. self._albums)
         if isinstance(other, self.__class__):
@@ -1163,5 +1348,19 @@ class PhotoInfo:
         return False
 
     def __ne__(self, other):
-        """ Compare two PhotoInfo objects for inequality """
+        """Compare two PhotoInfo objects for inequality"""
         return not self.__eq__(other)
+
+    def __hash__(self):
+        """Make PhotoInfo hashable"""
+        return hash(self.uuid)
+
+
+class PhotoInfoNone:
+    """mock class that returns None for all attributes"""
+
+    def __init__(self):
+        pass
+
+    def __getattribute__(self, name):
+        return None
