@@ -6,6 +6,7 @@ import plistlib
 from .._constants import (
     _PHOTOS_5_MODEL_VERSION,
     _PHOTOS_6_MODEL_VERSION,
+    _PHOTOS_7_MODEL_VERSION,
     _TESTED_DB_VERSIONS,
 )
 from ..utils import _open_sql_file
@@ -73,12 +74,12 @@ def get_db_model_version(db_file):
 
     model_ver = get_model_version(db_file)
     if _PHOTOS_5_MODEL_VERSION[0] <= model_ver <= _PHOTOS_5_MODEL_VERSION[1]:
-        db_ver = 5
+        return 5
     elif _PHOTOS_6_MODEL_VERSION[0] <= model_ver <= _PHOTOS_6_MODEL_VERSION[1]:
-        db_ver = 6
+        return 6
+    elif _PHOTOS_7_MODEL_VERSION[0] <= model_ver <= _PHOTOS_7_MODEL_VERSION[1]:
+        return 7
     else:
         logging.warning(f"Unknown model version: {model_ver}")
         # cross our fingers and try latest version
-        db_ver = 6
-
-    return db_ver
+        return 7
